export const panel_models_icon_ToggleIcon = "bk-panel-models-icon-ToggleIcon"
export const IconLabel = "bk-IconLabel"
export const IconRow = "bk-IconRow"
export default `:host(.bk-panel-models-icon-ToggleIcon){transition:opacity transform 0.3s ease-in;}:host(.bk-panel-models-icon-ToggleIcon:not(.active)){opacity:0.8;}:host(.bk-panel-models-icon-ToggleIcon:hover){opacity:1;transform:scale(1.1);}.bk-IconLabel{margin-left:5px;}.bk-IconRow{display:flex;align-items:center;justify-content:center;}`
