export { VTKJSPlot } from "./vtkjs";
export { VTKVolumePlot } from "./vtkvolume";
export { VTKAxes } from "./vtkaxes";
export { VTKSynchronizedPlot } from "./vtksynchronized";
//# sourceMappingURL=index.d.ts.map