declare module "styles/button.css" {
  export const btn_group: string
  export const btn: string
  export const btn_default: string
  export const active: string
  export const btn_primary: string
  export const btn_success: string
  export const btn_warning: string
  export const btn_danger: string
  export const btn_light: string
  export const panel_models_widgets_FileDownload: string
  export const TablerIcon: string
  export default ""
}
